commit d1092dd861b6e9275b4673b43c04c699a0629da1
Author: Kirill Astashov <badblock@sarov.info>
Date:   Wed Sep 13 16:07:49 2017 +0930

    forum: Implement attachments in subfolders

diff --git a/forum/includes/functions_download.php b/forum/includes/functions_download.php
index e00be1e..e44f0f7 100644
--- a/forum/includes/functions_download.php
+++ b/forum/includes/functions_download.php
@@ -128,6 +128,52 @@ function send_file_to_browser($attachment, $upload_dir, $category)
 
 	$filename = $phpbb_root_path . $upload_dir . '/' . $attachment['physical_filename'];
 
+    // BadBlock start.
+    if (preg_match("/^\d+_(.*)$/", $attachment['physical_filename'], $pockets) || preg_match("/^thumb_\d+_(.*)$/", $attachment['physical_filename'], $pockets)) {
+        $subdir1 = substr($pockets[1], 0, 2);
+        $subdir2 = substr($pockets[1], 2, 2);
+        $filename = $phpbb_root_path . $upload_dir . "/{$subdir1}/{$subdir2}/" . $attachment['physical_filename'];
+        $filedir = $phpbb_root_path . $upload_dir . "/{$subdir1}/{$subdir2}";
+        $oldfilename = $phpbb_root_path . $upload_dir . '/' . $attachment['physical_filename'];
+        if (!@file_exists($filename))
+        {
+            if (!@file_exists($oldfilename)) {
+                send_status_line(404, 'Not Found');
+                trigger_error('ERROR_NO_ATTACHMENT');
+            }
+            if (!is_dir($filedir)) {
+                @mkdir($filedir, 0777, true);
+            }
+            if (!@rename($oldfilename, $filename)) {
+                send_status_line(404, 'Not Found');
+                trigger_error('ERROR_NO_ATTACHMENT');
+            }
+            // Fix image orientation.
+            if (function_exists('exif_read_data')) {
+                $exif = exif_read_data($filename);
+                if (!empty($exif['Orientation'])) {
+                    $image = imagecreatefromjpeg($filename);
+                    switch ($exif['Orientation']) {
+                        case 3:
+                            $image = imagerotate($image, 180, 0);
+                            break;
+
+                        case 6:
+                            $image = imagerotate($image, -90, 0);
+                            break;
+
+                        case 8:
+                            $image = imagerotate($image, 90, 0);
+                            break;
+                    }
+
+                    imagejpeg($image, $filename, 90);
+                }
+            }
+        }
+    }
+    // BadBlock end.
+
 	if (!@file_exists($filename))
 	{
 		send_status_line(404, 'Not Found');
diff --git a/forum/phpbb/attachment/delete.php b/forum/phpbb/attachment/delete.php
index 922f24b..d0d0ee5 100644
--- a/forum/phpbb/attachment/delete.php
+++ b/forum/phpbb/attachment/delete.php
@@ -412,6 +412,14 @@ class delete
 		}
 
 		$filename = ($mode == 'thumbnail') ? 'thumb_' . utf8_basename($filename) : utf8_basename($filename);
+        // BadBlock start.
+        // Delete file in new location, old location will be cleared below by core.
+        if (preg_match("/^\d+_(.*)$/", $filename, $pockets) || preg_match("/^thumb_\d+_(.*)$/", $filename, $pockets)) {
+            $subdir1 = substr($pockets[1], 0, 2);
+            $subdir2 = substr($pockets[1], 2, 2);
+            $this->filesystem->remove($phpbb_root_path . $this->config['upload_path'] . "/{$subdir1}/{$subdir2}/" . $filename);
+        }
+        // BadBlock end.
 		$filepath = $this->phpbb_root_path . $this->config['upload_path'] . '/' . $filename;
 
 		try
